function [sys,x0,str,ts] = RIV(t,X,U,flag)
%
% RIV pour l'identification d'un ARX(2,2) i.e.
%    y(t)+a(1)y(t-1)+a(2)y(t-2)=b(1)u(t-1)+b(2)u(t-2)
%
switch flag,
  case 0
    [sys,x0,str,ts] = mdlInitializeSizes; % Initialization
  case 2
    sys = mdlUpdate(t,X,U); % Update discrete states
  case 3
    sys = mdlOutputs(t,X,U); % Calculate outputs
  case {1, 4, 9} % Unused flags
    sys = [];
  otherwise
    error(['unhandled flag = ',num2str(flag)]); % Error handling
end
% End of dsfunc.
%==============================================================
% Initialization
%==============================================================
function [sys,x0,str,ts] = mdlInitializeSizes
% Call simsizes for a sizes structure, fill it in, and convert it 
% to a sizes array.
sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 4+4+4+10+1;
sizes.NumOutputs     = 4;
sizes.NumInputs      = 2;
sizes.DirFeedthrough = 0; % Matrix D is empty.
sizes.NumSampleTimes = 1;
sys = simsizes(sizes); 
X0=zeros(4+4+4,1);
P0=100*eye(4);
x0=[X0;SymReshape(P0);0];   % Initialize the discrete states.
str = [];                   % Set str to an empty matrix.
ts  = [1 0];                % sample time: [period, offset]
% End of mdlInitializeSizes.
%==============================================================
% Update the discrete states
%==============================================================
function sys = mdlUpdate(t,X,U)
theta=X(1:4);
phi=X(5:8)';
zeta=X(9:12)';
P=SymReshape(X(13:22));
RIVok=X(23);
y=U(1);
u=U(2);
if RIVok==0,
    den=0.1+phi*P*zeta';
    if den<1e-3,
        disp(['Warning in RIV at time t=',num2str(t),': Instrumental variables produces singular value']);
        RIVok=20;
        disp(['Switch to RLS for ',num2str(RIVok),' sampled times']);
        theta=zeros(4,1);
        P=100*eye(4);
    end,
end,
if RIVok==0, % RIV is fine
    L=P*zeta'/den;
    P=(eye(4)-L*zeta)*P;
    theta=theta+L*(y-phi*theta);
    phi=[-y;phi(1);u;phi(3)];
    zeta=[-zeta*theta;zeta(1);u;zeta(3)];
    sys = [theta;phi;zeta;SymReshape(P);0];
else, % RLS is applied
    L=P*phi'/(0.1+phi*P*phi');
    P=(eye(4)-L*phi)*P;
    theta=theta+L*(y-phi*theta);
    phi=[-y;phi(1);u;phi(3)];
    zeta=phi;
    if RIVok==1,
        disp(['Switch back to RIV at time ',num2str(t)]);
    end,
    sys = [theta;phi;zeta;SymReshape(P);RIVok-1];
end,
% End of mdlUpdate.
%==============================================================
% Calculate outputs
%==============================================================
function sys = mdlOutputs(t,X,u)
sys = X(1:4);
% End of mdlOutputs.